#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Created On: 2023
# Created By: S. Buarque
# Assistance: This script was created from examples given on,
# https://github.com/rguilcas/climate-python-tips/blob/master/seaborn_presentation_RGuillaume_v2.ipynb
#
# Notes:
#
# Visualisation statistiques de données --> seaborn
#----- Outil construit sur matplotlib
#----- Visualisation complexes (très rapide)
#----- Calcul statistiques simples
#
# Installer anaconda et créer un environnement afin de pouvoir en profiter pleinement. 
# Pour ce faire suivez les instructions ici: https://www.anaconda.com/
#
# Une fois anaconda installé, tapez:
#
# conda create -n seaborn-environment
# conda install -c conda-forge seaborn matplotlib numpy
# conda activate seaborn-environment
#
# Données sous la forme de dataframe (pandas) ou alors listes python
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

from __future__ import division, print_function, unicode_literals, absolute_import

import csv
from collections import defaultdict
import pprint

import os,sys
import numpy as np

import seaborn as sns

import matplotlib.pyplot as plt
import matplotlib
print('matplotlib.__version__ : ',matplotlib.__version__)

import pandas as pd

#-------------------------------
# Default Parameters
#-------------------------------
#
dir_out='./tmp/'

varname = 'snct' 
varlongname = 'Snow Covered Fraction' 

varname = 'snd' 
varlongname = 'Snow Depth'

twin = '1979-2018'
yrStrt=int(twin.split('-')[0])
yrLast=int(twin.split('-')[1])+1
#
# Define lists
lexpes = ["CROCUS-ERA5", "CROCUS-ERAI" ]
lregions = ["Eurasia", "NAmerica" ]

opts = [ '[ JAN FEB ... ]' ]
lenmin = len(opts)+1
_usage = "Usage: %prog " + str(opts[0:lenmin])
if len(sys.argv) == lenmin :  
    month = sys.argv[1]
else:
    print("\n + + + + + + + + + + +")
    print("\n %s",_usage)
    month='MAY'
    print("\n Default parameters are: month =",month )
    print("\n + + + + + + + + + + +")


# Input file
dir_data = './ana/'
input_data = "stats_"+lexpes[0]+"_"+varname+"_"+month+"_"+twin+"_"+lregions[0]+".dat"
single_csvfile = dir_data+input_data
print(single_csvfile)
##
#
table = []
try:
    with open(single_csvfile) as csvfile:
        reader = csv.DictReader(csvfile)
        headers = reader.fieldnames
        for row in reader:
            table.append(row)
except FileNotFoundError:
    print("File not found.")
    sys.exit()

tmp_list=list(table[0].keys())
tmp_list += ['region']
tmp_list += ['simu']
##
#
field_names = ["year", "min", "max", "ave", "stdev", "skewness", "kurtosis"]
field_names += ['region']
field_names += ['simu']
declared_field_names = ["year", "min", "max", "ave", "stdev", "skewness", "kurtosis", "region", "simu"]
print('\n', type(field_names), *field_names)
print('\n', type(declared_field_names), *declared_field_names)
#
loc_input_file = 'stats_'+varname+'_'+month+'_'+twin+'.dat'
print("\n Creation of the synthetical file : ",loc_input_file,"\n")
os.system("if [ -f "+loc_input_file+" ] ; then rm -f "+loc_input_file+" ; fi")
os.system("touch "+loc_input_file)
#
#
for expe in lexpes  :
  fstat = 'stats_'+expe+'_'+varname+'_'+month+'_'+twin+'.dat'
  print(" Creation of a new file : ",fstat)
  os.system("if [ -f "+fstat+" ] ; then rm -f "+fstat+" ; fi")
  os.system("touch "+fstat)
  for region in lregions :
      tmpfile = dir_data+"stats_"+expe+"_"+varname+"_"+month+"_"+twin+"_"+region+".dat"
      cmd = "tail -n +2 "+tmpfile+" | awk -v var="+region+" '{print $0,var}' >> stats_"+expe+"_"+varname+"_"+month+"_"+twin+".dat"
      os.system(cmd)
  os.system("cat "+fstat+"| awk -v varexp=\""+expe+"\" '{print $0\" \"varexp}' >> "+loc_input_file)

# Defining the dict and passing 
rep = defaultdict(lambda: defaultdict(lambda: dict))

with open(loc_input_file, "r") as input_fic:
	content = csv.DictReader(input_fic, fieldnames=field_names, delimiter=" ", skipinitialspace=True)
	for row in content:
		rep[(row["simu"], row["region"])][int(row["year"])] = float(row["ave"])

# Climatology
for my_simulation in lexpes :
    for my_region in lregions :
        val=list(rep[(my_simulation, my_region)].values())
        keys=list(rep[(my_simulation, my_region)].keys())
        fakeclm=sum(val[0:20])/len(val)
        times = pd.date_range(keys[0], keys[-1], name='time')
        # Must convert the List to a DataFrame
        anomaly= pd.DataFrame(val) - fakeclm

df_csv = pd.DataFrame(rep)
"""
Reading as csv dataframe (pandas).
"""
f = open(loc_input_file, 'r')
data = []
# Loop over lines and extract variables of interest
for line in f:
    line = line.strip()
    columns = line.split()
    source = {}
    source['year'] = columns[0]
#    source['min'] = float(columns[1])
#    source['max'] = float(columns[2])
    source['ave'] = float(columns[3])
    source['stdev'] = float(columns[4])
# skewness = symmetry, normal distribution =0 (perfectly symmetrica)
#    source['skewness'] = float(columns[5])
# kurtosis (ou Pearson) = flatness, compared to a normal distributuion 
# K > 0 : data are more clustered in the center and have thinner ends
# = moment centré d'ordre 4 ramené à un nombre sans dimension, en le dividant par le carré de la variance. 
#    source['kurtosis'] = float(columns[6])
    source['region'] = columns[7]
    source['simu'] = columns[8]
    source
    data.append(source)
    type(source)

df = pd.DataFrame(data=data)
f.close()
##
#
# Extract column values by using DataFrame.loc[] property.
clm = {}
for my_region in lregions :
    df1=df.loc[df['region'] == my_region, 'simu']
    val=df[df['region'] == my_region]['ave']
    ival=int(len(val)/2)-1
    simuA = df.loc[df['region'] == my_region, 'simu'].iloc[ival]
    simuB = df.loc[df['region'] == my_region, 'simu'].iloc[ival+1]
    subsetA=sum(val[0:ival])/ival
    subsetB=sum(val[ival+1:len(val)])/ival
    clm[my_region]=[simuA, subsetA, simuB, subsetB]
    #clm.setdefault(my_region, []).append(my_simulation)
    print(my_region, clm[my_region][0], clm[my_region][1])
    print(my_region, clm[my_region][2], clm[my_region][3])


#-------------------------------
# Figures
#-------------------------------
x_tick_interval = 1
value_tick = np.arange(yrStrt,yrLast,x_tick_interval)
header_name = np.arange(yrStrt,yrLast)
header_name_sel = [header_name[idx] for idx in  np.arange(0,len(header_name),x_tick_interval)]
print("\n header_name_sel : ",header_name_sel,"\n")
palette='Greys'
#
####
nf=1
print("\n + + + + + + + + + + +\n plot("+str(nf)+")")
#
sns.set(style="ticks", palette=palette)
sns.set_theme(style="ticks", palette=palette)
g=sns.catplot(x="simu", y="ave", col="region", 
                 data=df, kind="box",
                 aspect=0.8, height=4, width=0.8,
                 fliersize=2.5,linewidth=1.1, notch=False,orient="v")
g.set(xlabel=month, ylabel=varlongname)

# set y limits
# May
if month == "MAY" :
    g.axes[0,0].set_ylim((0,.4))
    g.axes[0,1].set_ylim((0,.4))
if month == "APR" :
    g.axes[0,0].set_ylim((.2,.7))
    g.axes[0,1].set_ylim((.2,.7))

# Additional spines(small titles) can be removed by setting left=True or bottom=True
# sns.despine: the spines on the top and right side of a figure are removed
sns.despine()     # must be write here
g.savefig(dir_out+'plot'+str(nf)+'_'+month+'_'+twin+'.png')
plt.clf()
#
####
nf+=1
print("\n + + + + + + + + + + +\n plot("+str(nf)+")")
sns.set(style = 'whitegrid') 
g = sns.displot( data=df,
            x = 'ave',
            y = 'stdev',
            hue='simu', hue_order=lexpes,
            col='region',
            kind='kde',  #scatter,kde,hist
            fill=False,  # =False, contours only
            col_wrap = 2,
            palette= sns.set_palette(['#000000', '#DAA520']),
            height=4., aspect=1.2,
            facet_kws=dict(margin_titles=True, sharex=False, sharey=False)
            )
g.set(ylim=(0.1, .4))
g.set(xlim=(0., .8))

# To put additional title, but its incompatible with
# sns.despine, either one or the other
# g.set_titles(col_template=month)

# To move the legend if any. Default of title is 'simu', to remove it put title=''.
sns.move_legend(obj=g, title='', loc='lower left', bbox_to_anchor=(0.37, 0.8), frameon = True)

if month != "APR" :
    # To remove legend to panel (for the second figure)
    g._legend.remove()

plt.subplots_adjust(hspace = 2.6)
sns.despine()     # must be write here
plt.savefig(dir_out+'plot'+str(nf)+'_'+month+'_'+twin+'_displot.png')
plt.clf()
#
sys.exit()
