load "fct_res_crcPolar.ncl"
;
; Created On: 2023
; Created By: S. Buarque
;
;=================================================================
undef("get_res")
function get_res()
begin
; common resources
  res = fct_res_crcPolar
  ;res@gsnPaperOrientation    = "seascape"
  ;res@vpWidthF      = 0.95
  res@vpHeightF     = 0.39
  ;res@vpXF          = 0.03
  res@mpOceanFillColor       = "white" ;62           ; array index in color map
  res@mpLandFillColor        = "grey11" ;8          ; for land, ocean, and inland h20
  res@mpInlandWaterFillColor = 62
  res@mpCenterLonF           = 0  
  res@tmXTOn                 = False
  res@tmYROn                 = False
  res@gsnCenterString        = ""
  res@gsnStringFontHeightF   = 0.012
  res@gsnLeftString          = "" ;expe_graph(0)

  res@tmXBLabelFontHeightF    = 0.008
  res@tmXBOn           = True
  return res
end
;
;=================================================================
undef("get_gsres")
function get_gsres()
begin
   gsres                    = True
   gsres@gsMarkerThicknessF = 2.
   gsres@tmYROn             = False          
   gsres@tmXTOn             = False      
   gsres@tmYLOn             = False          
   gsres@tmXBOn             = False        

   gsres1                    = gsres
   gsres1@gsMarkerIndex      = 4.
   gsres1@gsMarkerSizeF      = 0.0045
   gsres1@gsMarkerColor      = "grey18"   ;11"

   gsres@gsMarkerIndex      = 16.
   gsres@gsMarkerSizeF      = 0.0028
   gsres@gsMarkerColor      = "black" 
  return ( [/ gsres, gsres1 /] )
end
;
;=================================================================
undef("get_lbres")
function get_lbres()
begin
  lbres                           = True
  lbres@lbAutoManage              = False
  lbres@lbPerimOn                 = False         ; Turn off labelbar perimeter.
  lbres@lbMonoFillPattern         = True  
  lbres@lbOrientation             = "Horizontal"
  lbres@vpWidthF             = 0.35                 ; size
  lbres@vpHeightF            = 0.06
  lbres@lbLabelAlignment     = "InteriorEdges"     ; where to label
  lbres@lbLabelFontHeightF   = 0.011    
  lbres@lbBoxEndCapStyle     = "TriangleBothEnds"
  return lbres
end
;
;=================================================================
undef("listpush_main")
function listpush_main()
begin
;--Create a list with variables to be processed
; Options to NewList:  "fifo" -> (First In, First Out),  "lifo" -> (Last In, First Out))
 list_varnames = NewList("fifo")
;
;--Collective processing: common variables 
 duree     = (/"DUREE_CONT_bias","(a) Duration of continuous snow","days since 1st August"/)
 avg_sd    = (/"AVG_SD_bias","(a) Annual average of snow depth","m"/)
 duree_tot = (/"DUREE_TOT_bias","(a) Total duration","days since 1st August"/)
 date_max  = (/"DATE_MAX_SD_bias","(b) Maximum date of snowfall ","days since 1st August"/)
 date_deb  = (/"DATE_DEB_bias","(c) First day of continuous snow", "days since 1st August"/)
 date_fin  = (/"DATE_FIN_bias","(d) Last day of continuous snow","days since 1st August"/)
;
 ListPush(list_varnames,date_fin)
 ListPush(list_varnames,date_deb)
 ListPush(list_varnames,date_max)
 ListPush(list_varnames,duree)

 return list_varnames
end 
;
;=================================================================
undef("single_plots")
function single_plots(wks,variable,nv,list_varnames,lat_s_1d,lon_s_1d,PeriodString,nespace)
local plot
begin
  echelle  = (/-30,-25,-20,-15,-10,-5,0,5,10,15,20,25,30,55555555/)
  colors   = (/129,120,113,101,94,86,78,70,55,46,33,21,12,2/)
  labels   = (/"-20","-15","-10","-5","-2","-1","0","1","2","5","10","15","20"/)
  labavg   = (/"-.1","-.05","-.04","-.03","-.02","-.01","0",".01",".02",".03",".04",".05",".1"/)
  nboxes = dimsizes(echelle)                  ; # of labelbar boxes

  MainTitle = list_varnames[nv](1)
  title = PeriodString+" ("+list_varnames[nv](2)+") "

  var_1d = ndtooned(variable)

  indi = ind(.not.(ismissing(var_1d)))
  nstation = dimsizes(indi)
  lati_s = lat_s_1d(indi)
  loni_s = lon_s_1d(indi)

  tmp = onedtond(var_1d,(/2,nespace/))
  data = tmp(:,indi)

  result = get_gsres
  gsres = result[0]
  gsres1 = result[1]

  lbres = get_lbres

  YY = (/0.95,0.95,0.45,0.45/)
  XX = (/0.05,0.50,0.05,0.50/)

  res= get_res()
  res@vpYF                   = YY(nv)
  res@vpXF                   = XX(nv)
  plot = gsn_csm_map_polar(wks,res)        ; draw global map

; -- Putting stations outline
  do i=0,nstation-1
     gsn_polymarker(wks,plot,loni_s(i),lati_s(i),gsres1)
  end do

; -- Putting colors within the circle
   do i=0,nstation-1
       do j=0,dimsizes(echelle)-1
          if (.not.(ismissing(data(0,i)))).and.(data(0,i).le.echelle(j)) then
              gsres@gsMarkerColor      = colors(j)
             j=dimsizes(echelle)-1+3
          end if
       end do
    gsn_polymarker(wks,plot,loni_s(i),lati_s(i),gsres)
   end do

  txres               = True
  txres@txFontHeightF = 0.02
  ;gsn_text_ndc(wks,"MinMaxString",0.50,0.17,txres)         ; gsn_text_ndc(wks,text,x,y,res)  

  XG = XX(nv)+0.02		; left/right
  YG = YY(nv)-0.41

  lbres@lbFillColors              = colors
  if .not.ismissing(str_match(MainTitle,"average")) then
  	gsn_labelbar_ndc(wks,nboxes,labavg,XG,YG+0.01,lbres)
  else
  	gsn_labelbar_ndc(wks,nboxes,labels,XG,YG+0.01,lbres)
  end if
  delete(lbres@lbFillColors)

  txres@txFontHeightF = 0.012  
  gsn_text_ndc(wks,MainTitle,XG+0.17,YY(nv)+0.04,txres) 
  txres@txFontHeightF = 0.01  
  gsn_text_ndc(wks,title,XG+0.17,YY(nv)+0.02,txres) 

  return plot
end
;
;
begin
space = " "
year1=1950
year2=2012 
PeriodString=tostring(year1)+"-"+tostring(year2)
exper="CROCUS-ISBA_land-hist_r1i1p1f2"
exper_graph="CROCUS-ERA5"
outdir_graphes="./tmp/"
seuil=0.01
hauteur_seuil=100.
outdir_save_simu="./dat/"
numgraphe=2


   list_varnames = listpush_main()
   nvars = ListCount(list_varnames)   ; Get total list count

   subtitle      = "OBS-EXP"
   expe        = str_split(exper," ")
   expe_graph  = str_split(exper_graph," ")
   outdir_savesimu = str_split(outdir_save_simu," ")
   num_graphe = sprinti("%0.2i",numgraphe)

   print("   ")
   print("Reading data from "+expe(0))
   chem_in = "fic_2D_stat_comp_"+expe(0)+"_obs_"+year1+"_"+year2+"_seuil_"+seuil+"_diff_alti_"+hauteur_seuil+".m_test.nc"
   gg  = addfile(chem_in,"r")
   lat_s = gg->latitude
   lon_s = gg->longitude
   nlat = dimsizes(lat_s)
   nlon = dimsizes(lon_s)
   nespace = nlat*nlon
   variable = new((/nlat,nlon/),float)

   lat2d = new((/nlat,nlon/),float)
   lon2d = new((/nlat,nlon/),float)
   do j=0,nlat-1
      lon2d(j,:) = lon_s
   end do
   do i=0,nlon-1
      lat2d(:,i) = lat_s
   end do
   lat_s_1d = ndtooned(lat2d)
   lon_s_1d = ndtooned(lon2d)  

;--------------GRAPHIQUE-----------------------

  pltName = outdir_graphes+exper_graph+"_p"+num_graphe+"_indicateurs_moyenne_1D_obs_stations_selec_base_1er_aout_"+year1+"_"+year2
  system("rm -f "+pltName)
  wks = gsn_open_wks("png",pltName)
  ;gsn_define_colormap(wks,"gsdtol")
  gsn_define_colormap(wks,"MPL_RdYlBu")

  plot = new(4,graphic)   
  do nv=0, nvars-1
     name_variable_exp = list_varnames[nv](0)
     variable = gg->$name_variable_exp$
     print(""+name_variable_exp)
     plot(nv) = single_plots(wks,variable,nv,list_varnames,lat_s_1d,lon_s_1d,PeriodString,nespace)
     draw(plot(nv))
  end do
  frame(wks)

end
