load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;*************************************************
undef("goodunits")
function goodunits(tmp,varname,newunits)
local tmp, varname, scale, offset 
begin
 setfileoption("nc","MissingToFillValue",False)
 cr= tochar(10)
 scale = 1.
 offset = 0.
;
 print("****************************************")
 print("** Within goodunits **")
 printMinMax(tmp,0)
;
; Rappel:  [W] = [J s-1]        [J] = [N m]       [N] = [kg m s-1]
; Precipitation will be expressed in multiple of meters (cm, mm,...) of liquid water equivalent for time interval (yr). 
; --> 1 mm of rain corresponds to 1 l of water /m2  or approximately 10 mm of snow /m2.
;
 nb_sec_hh   =  60.*60.
 nb_sec_dd   =  24.*nb_sec_hh
 nb_sec_yy   =  365.*nb_sec_dd
 rho         =  1000.                     ; kg m-3 (water equivalent)

;  
 if .not.isatt(tmp,"units") then tmp@units="None" end if
 chain = str_split(tmp@units,"-")
 if .not.ismissing(str_match(tmp@units,"1")) .and. dimsizes(chain).lt.2 then tmp@units="None" end if
;
 if typeof(tmp).ne."short" .and. isatt(tmp,"missing_value") then delete(tmp@missing_value) end if
 if typeof(tmp).ne."short" .and. .not.isatt(tmp,"_FillValue") then tmp@_FillValue = -999. end if
;
 if(tmp@units .eq. "None" ) then
    print("   NO units ")
;--- "units" consistent with seasonal mean process: ~/tong/src/pybox/d_smb/4-savef_smb_MperY_monthly_dexp_W.py
    if .not.ismissing(str_match(varname,"Albedo")) then
       if max(tmp).gt.100 then  
          scale = 0.01
          offset = 0.
          goodUnit = ""
       end if
    end if
    if .not.ismissing(str_match(varname,"sfc_BBf")) then  
        goodUnit = "mWE/yr" 
    end if
    if .not.ismissing(str_match(varname,"emperature")) then
       offset = -273
       goodUnit = "Degrees C"
    end if
    if varname.eq."ps" .or.  varname.eq."psl"  then 
       scale = 0.01
       goodUnit = "hPa"
    end if
    if varname.eq."pr"  .or.  varname.eq."prc"  .or.  varname.eq."prls"  .or.  \
       varname.eq."tp"  .or.  varname.eq."cp"   .or.  varname.eq."lsp"  then 
;   [ kg m-2 s-1 ] --> mmWE : [ kg m-2 s-1 ] / [ kg m-3 ] = [m s-1 ]  
       tmp = where(tmp.gt.1.e+03 .or. tmp.lt.1.e-05, tmp@_FillValue, tmp)
       if newunits.eq."mm/h" .or. newunits.eq."mmWE/h" then             
          scale= (1./rho)*(1.e+03*nb_sec_hh)   
          goodUnit = "mmWE/h"   ; --->  [m s-1 ] -->  [ 1.e+3 mm * 3600 h-1 ]
       end if
       if newunits.eq."mm/d" .or. newunits.eq."mmWE/d"  then
          scale= (1./rho)*(1.e+03*nb_sec_dd)
          goodUnit = "mmWE/d"   ; --->  [m s-1 ] -->  [ 1.e+3 mm * 86400 d-1 ]
       end if
       if newunits.eq."m/yr" .or. newunits.eq."mWE/yr"  then
          scale= (1./rho)*nb_sec_yy            
          goodUnit = "mWE/yr"   ; --->  [m s-1 ] -->  [ * nb_sec_yy yr-1 ] 
       end if
       if newunits.eq."mm/yr" .or. newunits.eq."mmWE/yr" then
          scale= (1./rho)*(1.e+03*nb_sec_yy);  [m s-1 ] -->  [ 1.e+3 mm * nb_sec_yy yr-1 ] 
          goodUnit = "mmWE/yr"
       end if
    end if
;
 else 
    print("   WITH units --> "+ tmp@units+"  (to "+newunits+")")
    if tmp@units.eq.newunits then  
       print("  units ("+tmp@units+")   .eq.   newunits("+ newunits+")" )
       if tofloat(max(tmp)) .gt. 273 then
          offset = -273
          goodUnit = "Degrees C"
       end if
    else
       if tmp@units .eq. "K" .or. tmp@units .eq. "Degrees K"  then
          offset = -273
          goodUnit = "Degrees C"
       end if
       if tmp@units .eq. "Pa"  then
          scale = 0.01
          goodUnit = "hPa"
       end if
       if (tmp@units.eq."kg m-2")  then
             if newunits.eq."m" then 
                scale= (scale/rho)
             end if
             if newunits.eq."cm" then 
                scale= (scale/rho)*1.e+02
             end if
             if newunits.eq."mm" then 
                scale= (scale/rho)*1.e+03
             end if
             goodUnit = newunits 
       end if
       if (tmp@units.eq."kg/m2/s")  .or.  (tmp@units.eq."kg/m^2/s")  .or.  (tmp@units.eq."kg m-2 s-1")  then
;      [ kg m-2 s-1 ] --> mmWE : [ kg m-2 s-1 ] / [ kg m-3 ] = [m s-1 ]  
          if newunits.eq."mm/h" .or. newunits.eq."mmWE/h" then             
             scale= (1./rho)*(1.e+03*nb_sec_hh)
             goodUnit = "mmWE/h"  ; ---> [m s-1 ] -->  [ 1.e+3 mm * 3600 h-1 ]
          end if
          if newunits.eq."mm/d" .or. newunits.eq."mmWE/d"  then
             scale= (1./rho)*(1.e+03*nb_sec_dd)
             goodUnit = "mmWE/d"  ; ---> [m s-1 ] -->  [ 1.e+3 mm * 86400 d-1 ]
          end if
          if newunits.eq."m/yr" .or. newunits.eq."mWE/yr"  then
             scale= (1./rho)*nb_sec_yy            
             goodUnit = "mWE/yr"  ; ---> [m s-1 ] -->  [ * nb_sec_yy yr-1 ] 
          end if
          if newunits.eq."mm/yr" .or. newunits.eq."mmWE/yr" then
             scale= (1./rho)*(1.e+03*nb_sec_yy);  [m s-1 ] -->  [ 1.e+3 mm * nb_sec_yy yr-1 ] 
             goodUnit = "mmWE/yr"
          end if
       end if
;... Case of ERA-Interim's precipitation (tmp@units='m') IS NOT in [m] but in [1.e+2 kg m-2 s-1]
       if tmp@units.eq."m" 
          scale = 1.e-02                                          ; [1.e+2 kg m-2 s-1]  <-->  [kg m-2 s-1]
          if newunits.eq."m/yr"  then
                scale= scale*(nb_sec_yy)/rho          ;  then [ * nb_sec_yy (yr-1) ] --> m/yr 
                goodUnit = "mWE/yr"
          end if
          if newunits.eq."mm/h"  then
                scale= scale*(1.e+03*nb_sec_hh)/rho; [ then [ 1.e+3 mm * 3600 h-1 ] 
                goodUnit = "mmWE/h"
          end if
       end if
;... Case of MODGRNLD data
       if tmp@units.eq."cm" .and. newunits.eq."mm" then 
                scale = 1.e+01
                goodUnit = "mm"
       end if
    end if
 end if
 if varname.eq."ua"  .or.  varname.eq."va"  then
    goodUnit = "m/s"
 end if
;
 if newunits.ne.tmp@units then
    if newunits.ne.goodUnit then  print(" Warning!!! ("+newunits+")   .NE.   "+ goodUnit)  end if
     tmp=tmp*tofloat(scale)+tofloat(offset)
     tmp@units=goodUnit
 end if
 print("   scale="+scale+", offset="+offset)
 printMinMax(tmp,0)
 print("** End    goodunits **")
 print("****************************************")
 return(tmp)
end
